# Arduino et afficheur LCD 4 segment
![category](domotique)
![tag](electronique)
![tag](arduino)

On peut faire un montage pour afficher l'heure sur un  afficheur LCD 4 segments.

![201709122114_01.png](./upload/201709122114_01.png)

Pour cela nous branchons sur un arduino uni

- un module rtc pour avoir l'heure
- un afficheur lcd 4 digit


Pour fonctionner il faut utiliser de librairie:

- librairie *Wire.h* pour le bus I2C
- librairie *DS1307.h* pour communiquer avec le module
- la librairie *TimerOne.h*

La librairie *Wire.h* est intégré dans l'arduino IDE

On peut trouver la librairie *DS1307.h* sur le lien https://raw.githubusercontent.com/SeeedDocument/Grove-RTC/master/res/RTC_Library.zip
(il suffira de copier les fichier DS1307.h et DS1307.cpp au même niveau que notre
code)

On peut trouver la librairie *TimerOne.h* sur le lien https://storage.googleapis.com/google-code-archive-downloads/v2/code.google.com/arduino-timerone/TimerOne-r11.zip

Et enfin notre code

```c++
#include "TimerOne.h"
#include <Wire.h>
#include "DS1307.h"

DS1307 clock;//define a object of DS1307 class

//With this pin setup you can connect pin 1 through 12 on the display to pins 2-13 on Arduino
byte pins[16] = {2,3,4,5,6,7,8,9,10,11,12,13};
byte rows[4] = {6,8,9,12};
byte cols[8] = {11,7,4,2,1,10,5,3};

byte screen[8] = {0, 0, 0, 0, 0, 0, 0, 0};
volatile byte screenRow = 0;
volatile byte screenCol = 0;

byte numbers[11][7] = {
  {1,1,1,1,1,1,0}, //zero
  {0,0,0,0,1,1,0}, //one
  {1,1,0,1,1,0,1}, //two
  {1,0,0,1,1,1,1}, //three
  {0,0,1,0,1,1,1}, //four
  {1,0,1,1,0,1,1}, //five
  {1,1,1,1,0,1,1}, //six
  {0,0,0,1,1,1,0}, //seven
  {1,1,1,1,1,1,1}, //eight
  {1,0,1,1,1,1,1}, //nine
  {0,0,0,0,0,0,0}  //off
};

byte numbersinv[11][7] = {
  {1,1,1,1,1,1,0}, //zero
  {0,1,1,0,0,0,0}, //one
  {1,1,0,1,1,0,1}, //two
  {1,1,1,1,0,0,1}, //three
  {0,1,1,0,0,1,1}, //four
  {1,0,1,1,0,1,1}, //five
  {1,0,1,1,1,1,1}, //six
  {1,1,1,0,0,0,0}, //seven
  {1,1,1,1,1,1,1}, //eight
  {1,1,1,1,1,0,1}, //nine
  {0,0,0,0,0,0,0}  //off
};

byte digitinv[4] = {3,2,1,0};

void setup() 
{
  Serial.begin(9600);
  clock.begin();
  
  Timer1.initialize(100);
  for (int i = 0; i < 16; i++)
    pinMode(pins[i], OUTPUT);
  Timer1.attachInterrupt(doubleBuffer);
  doubleBuffer();
}

// interrupt routine
void doubleBuffer() {
  // reset the previous iteration
  digitalWrite(translatePin(rows[screenRow]), HIGH); // set previous off
  digitalWrite(translatePin(cols[screenCol]), LOW);  // set previous off
  // go to the next iteration...
  // go to the next screenCol, wrap if necessary
  screenCol++;
  if (screenCol >= 8) {
    screenCol = 0;
    // when screenCol wraps, go to the next screenRow, wrap if necessary
    screenRow++;
    if (screenRow >= 4) {
      screenRow = 0;
    }
  }
  // set this iteration
  if((screen[screenRow]>>screenCol)&B1 == B1) {
    digitalWrite(translatePin(rows[screenRow]), LOW);  // set this on
    digitalWrite(translatePin(cols[screenCol]), HIGH); // set this on
  } else {
    digitalWrite(translatePin(rows[screenRow]), HIGH); // set this off
    digitalWrite(translatePin(cols[screenCol]), LOW);  // set this off
  }

}

byte translatePin(byte original) {
  return pins[original - 1];
}

void allOFF() {
  for (int i = 0; i < 8; i++)
    screen[i]=0;
}

void on(byte row, byte column) {
    screen[column-1] |= (B1<<(row-1));
}

void off(byte row, byte column) {
    screen[column-1] &= ~(B1<<(row-1));
}

void displayNumbers(byte digit, byte value, byte sens) {
  if(sens > 1) {
    for (int i = 0; i < 7; i++) {
      if (numbers[value][i] == 1) on(i+1,digit+1);
        else off(i+1,digit+1);
    }
  } else {
    for (int i = 0; i < 7; i++) {
      if (numbersinv[value][i] == 1) on(i+1,digitinv[digit]+1);
        else off(i+1,digitinv[digit]+1);
    }
    
  }
}


void dot_on(byte dot)
{
  on(8,dot);
}

void dot_off(byte dot)
{
  off(8,dot);
}

void decimals(byte bottom, byte top) {
  if(bottom) on(8,2);
  else off(8,2);
  if(top) on(8,3);
  else off(8,3);
}

void loop() 
{
  clock.getTime();
  byte nums[4] = {10,  10, 10, 10 };
  if(clock.hour/10 > 0) nums[0]= clock.hour/10;
  nums[1]= clock.hour % 10;
  nums[2]= clock.minute / 10;
  nums[3]= clock.minute % 10;
  for (int digit = 0; digit < 4; digit++) {
    displayNumbers(digit, nums[digit], 2);
  }
  dot_on(1);
  delay(1000);
}```
